# python-aio-geojson-geonetnz-quakes

[![Build Status](https://img.shields.io/github/actions/workflow/status/exxamalte/python-aio-geojson-geonetnz-quakes/ci.yaml)](https://github.com/exxamalte/python-aio-geojson-geonetnz-quakes/actions/workflows/ci.yaml)
[![codecov](https://codecov.io/gh/exxamalte/python-aio-geojson-geonetnz-quakes/branch/master/graph/badge.svg?token=USA5MZU6D5)](https://codecov.io/gh/exxamalte/python-aio-geojson-geonetnz-quakes)
[![PyPi](https://img.shields.io/pypi/v/aio-geojson-geonetnz-quakes.svg)](https://pypi.python.org/pypi/aio-geojson-geonetnz-quakes)
[![Version](https://img.shields.io/pypi/pyversions/aio-geojson-geonetnz-quakes.svg)](https://pypi.python.org/pypi/aio-geojson-geonetnz-quakes)

This library provides convenient async access to the
[GeoNet NZ Quakes](https://api.geonet.org.nz/#quakes) feed.
 

## Installation
`pip install aio-geojson-geonetnz-quakes`

## Usage
See below for examples of how this library can be used. After instantiating a 
particular class - feed or feed manager - and supply the required parameters, 
you can call `update` to retrieve the feed data. The return value 
will be a tuple of a status code and the actual data in the form of a list of 
feed entries specific to the selected feed.

Status Codes
* _OK_: Update went fine and data was retrieved. The library may still 
  return empty data, for example because no entries fulfilled the filter 
  criteria.
* _OK_NO_DATA_: Update went fine but no data was retrieved, for example 
  because the server indicated that there was not update since the last request.
* _ERROR_: Something went wrong during the update

**Parameters**

| Parameter          | Description                               |
|--------------------|-------------------------------------------|
| `home_coordinates` | Coordinates (tuple of latitude/longitude) |
| `mmi`              | Request quakes that may have caused shaking greater than or equal to the [MMI](https://en.wikipedia.org/wiki/Modified_Mercalli_intensity_scale) value in the New Zealand region. Allowable values are -1..8 inclusive. Default: -1 is used for quakes that are too small to calculate a stable MMI value for. |

**Supported Filters**

| Filter            |                            | Description |
|-------------------|----------------------------|-------------|
| Radius            | `filter_radius`            | Radius in kilometers around the home coordinates in which events from feed are included. |
| Minimum Magnitude | `filter_minimum_magnitude` | Minimum magnitude as float value. Only events with a magnitude equal or above this value are included. |
| Time              | `filter_time`              | Time interval; only events with a reported origin time that falls within now and this past time interval are included. |

**Example**
```python
import asyncio
from aiohttp import ClientSession
from aio_geojson_geonetnz_quakes import GeonetnzQuakesFeed
async def main() -> None:
    async with ClientSession() as websession:    
        # Home Coordinates: Latitude: -41.2, Longitude: 174.7
        # MMI: 2
        # Filter radius: 200 km
        # Filter minimum magnitude: 2.5
        feed = GeonetnzQuakesFeed(websession, (-41.2, 174.7), mmi=2,
                                  filter_radius=200,
                                  filter_minimum_magnitude=2.5)
        status, entries = await feed.update()
        print(status)
        print(entries)
asyncio.get_event_loop().run_until_complete(main())
```

## Feed entry properties
Each feed entry is populated with the following properties:

| Name             | Description                                                                   | Feed attribute |
|------------------|-------------------------------------------------------------------------------|----------------|
| geometry         | All geometry details of this entry.                                           | `geometry`     |
| coordinates      | Best coordinates (latitude, longitude) of this entry.                         | `geometry`     |
| external_id      | The unique public identifier for this quake.                                  | `publicID`     |
| title            | Title of this entry.                                                          | `locality`     |
| attribution      | Attribution of the feed.                                                      | n/a            |
| distance_to_home | Distance in km of this entry to the home coordinates.                         | n/a            |
| time             | The origin time of the quake.                                                 | `time`         |
| depth            | The depth of the quake in km.                                                 | `depth`        |
| magnitude        | The summary magnitude for the quake.                                          | `magnitude`    |
| locality         | Distance and direction to the nearest locality.                               | `locality`     |
| mmi              | The calculated MMI shaking at the closest locality in the New Zealand region. | `mmi`          |
| quality          | The quality of this information: best, good, caution, deleted.                | `quality`      |


## Feed Manager

The Feed Manager helps managing feed updates over time, by notifying the 
consumer of the feed about new feed entries, updates and removed entries 
compared to the last feed update.

* If the current feed update is the first one, then all feed entries will be 
  reported as new. The feed manager will keep track of all feed entries' 
  external IDs that it has successfully processed.
* If the current feed update is not the first one, then the feed manager will 
  produce three sets:
  * Feed entries that were not in the previous feed update but are in the 
    current feed update will be reported as new.
  * Feed entries that were in the previous feed update and are still in the 
    current feed update will be reported as to be updated.
  * Feed entries that were in the previous feed update but are not in the 
    current feed update will be reported to be removed.
* If the current update fails, then all feed entries processed in the previous
  feed update will be reported to be removed.

After a successful update from the feed, the feed manager provides two
different dates:

* `last_update` will be the timestamp of the last update from the feed 
  irrespective of whether it was successful or not.
* `last_update_successful` will be the timestamp of the last successful update 
  from the feed. This date may be useful if the consumer of this library wants 
  to treat intermittent errors from feed updates differently.
* `last_timestamp` (optional, depends on the feed data) will be the latest 
  timestamp extracted from the feed data. 
  This requires that the underlying feed data actually contains a suitable 
  date. This date may be useful if the consumer of this library wants to 
  process feed entries differently if they haven't actually been updated.
